//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

/** \file
 * Functions related to hardware self-test.
 */

#include <stdint.h>
#include <stdbool.h>
#include "p2p.h"
#include "bsp.h"
#include "buttons.h"
#include "commands.h"
#include "sx1239.h"

void LEDTest(void);
void ButtonTest(void);
void SPITest(void);
void RTCCTest(void);

extern void sx1239_init(void);

/** Flag indicating that the hardware should be flashing all LEDs. */
bool flashLEDs = false;

/**
 * Main entry point for hardware testing. Calls each test function.
 */
void HDWSelfTests(void)
{    
    ButtonTest();
    LEDTest();
    RTCCTest();
    SPITest();

    p2p_lcd_clear();
    p2p_lcd_goto(0,0);
    p2p_lcd_puts("All HW Test Done");
    p2p_lcd_goto(0,1);
    p2p_lcd_puts("SW1 to Continue");
    while(1)
    {
        if( SW1_HELD )
        {
            break;
        }
    }

}

/**
 * Verify button operation.
 *
 * Operator is asked to push each button in turn. Once the software detects
 * a press on the requested button, it moves to the next.
 */
void ButtonTest(void)
{
    p2p_lcd_clear();
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts(" HDW Self Tests ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("  Button  Test  ");
    __delay_ms(500);
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts("  Button  Test  ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("   Press  SW1   ");
    while(1)
    {
        if( SW1_HELD )
            break;
    }
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("   Press  SW2   ");
    while(1)
    {
        if( SW2_HELD )
            break;
    }
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("   Press  SW3   ");
    while(1)
    {
        if( SW3_HELD )
            break;
    }
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("   Press  SW4   ");
    while(1)
    {
        if( SW4_HELD )
            break;
    }

    p2p_spi(P2P_CMD_TEST_PUSH_BUTTONS);
    while(1)
    {
        if( SW1_HELD )
            break;
    }
}

/**
 * Verify LED connections by flashing them.
 */
void LEDTest(void)
{
    p2p_lcd_clear();
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts(" HDW Self Tests ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("    LED Test    ");
    __delay_ms(500);
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts(" LEDs Flashing ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("SW1 to Continue ");

    p2p_spi(P2P_CMD_FLASH_LEDS);
    flashLEDs = true;
    while( !SW1_HELD );
    flashLEDs = false;
    LED1 = LED2 = LED3 = LED4 = LED_OFF;
    p2p_spi(P2P_CMD_STOP_FLASH_LEDS);
}

/**
 * Verify SPI connections.
 *
 * This is done by attempting to read the "version" register of the SX1239. If
 * the value is non-zero, we assume the bus works correctly.
 */
void SPITest(void)
{
    p2p_lcd_clear();
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts(" HDW Self Tests ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("    SPI Test    ");
    __delay_ms(500);
    p2p_lcd_clear();
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts("  SPI  Testing  ");
    //sx1239_init();
    if( sx1239_getVersion() != 0 )
    {
        p2p_lcd_goto(0, 1);
        p2p_lcd_puts("  Successful   ");
    }
    else
    {
        p2p_lcd_goto(0, 1);
        p2p_lcd_puts("      Fail      ");
    }
    __delay_ms(500);
}

/**
 * Verify RTCC operation.
 *
 * This function enables the RTCC oscillator and periodically requests the
 * the current time. If the time is updating regularly on the LCD, assume
 * the test is successful.
 */
void RTCCTest(void)
{
    p2p_lcd_clear();
    p2p_lcd_goto(0, 0);
    p2p_lcd_puts(" HDW Self Tests ");
    p2p_lcd_goto(0, 1);
    p2p_lcd_puts("   RTCC  Test   ");
    __delay_ms(500);
    p2p_spi(P2P_CMD_RTCC_TEST);
    while(1)
    {
       if( SW1_HELD )
           break;
    }
}