//
// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
//

#include "includes.h"
#include "Config.h"

/**
 * KeeLoq interrupt-based reception state machine.
 * Call on TMR0 overflow interrupts.
 */
void rxi(void) {
    // this routine gets called every time TMR0 overflows
    RFBit = PICtail_RAW_DATA; //each bit received is accessed in real time
    LATBbits.LATB1 ^= 1;
    T0IF = 0;
    TMR0 = DEFAULT_TMR0;

    XTMR++; // extended 16 long timer update
    if (RFFull) // avoid overrun
        return;

    switch (RFstate) // state machine main switch
    {
        case TRFUNO: // Have detected a high level waiting for falling edge
            
            if (RFBit == 0) // Read RF level
            {   // falling edge detected  ----+
                //                            |
                //   	                      +-------

                RFstate = TRFZERO; // Falling edge detected, go back to state TRFZERO
            } else { // still high
                RFcount--;
                if (RFcount >= HIGH_TO) // Check for timeouot
                { // If timeout, then check for what type of transmission detected.
                    if (BitCount < KNBIT) // Not a complete transmission so reset state machine
                    {
                        RFtype = 0;
                        RFstate = TRFSYNC;
                        Bptr = 0;
                        BitCount = 0;
                        break;
                    }
                    if ((BitCount >= KNBIT) & (BitCount < XNBIT)) // Is this standard Keeloq?
                    {
                        RFtype = KeeLoq;
                    }

                    if ((BitCount >= XNBIT) & (BitCount < ANBIT)) // Is this XTEA?
                    {
                        RFtype = XTEA;
                    }
                    if (BitCount >= ANBIT) // Is this AES?
                    {
                        RFtype = AES;
                    }

                    RFstate = TRFreset; // Reset State machine for next transmission
                    RFFull = 0x01; // Flag to let the main routine know we have a complete transmission

                }//if RFcount
            }//else
            break;

        case TRFZERO: // We've detected a low level, look for rising edge
            if (RFBit) // Read RF bit.
            {   // rising edge detected     +----
                //                          |
                //                      ----+

                RFstate = TRFUNO;
                RX_Buffer[Bptr] >>= 1; // Rotate
                if (RFcount >= 0) {
                    RX_Buffer[Bptr] += 0x80; // Shift in bit
                }
                RFcount = 0; // Reset length counter

                if ((++BitCount & 7) == 0)
                    Bptr++; // Advance one byte
                if (BitCount == (ANBIT + 10)) // maximum number of bits
                {
                    RFstate = TRFreset; // Finished receiving
                    RFFull = 0x1;
                    RFtype = AES;
                }
            } else { // still low
                RFcount++;
                if (RFcount >= LOW_TO) // too long low
                { // If timeout, then check for what type of transmission detected.

                    RX_Buffer[Bptr] >>= 1; // Rotate
                    if (RFcount >= 0) //shift in last receive bit
                    {
                        RX_Buffer[Bptr] += 0x80; // Shift in bit
                    }

                    if (BitCount < KNBIT) // Not a valid complete transmission so reset state machine
                    {
                        RFtype = 0;
                        RFstate = TRFSYNC;
                        Bptr = 0;
                        BitCount = 0;
                        break;
                    }
                    if (BitCount >= KNBIT & BitCount < XNBIT) // Is this standard Keeloq?
                    {
                        RFtype = KeeLoq;
                    }

                    if (BitCount >= XNBIT & BitCount < ANBIT) // Is this AES?
                    {
                        RFtype = XTEA;
                    }
                    if (BitCount >= ANBIT) // Is this XTEA?
                    {
                        RFtype = AES;
                    }

                    RFstate = TRFreset; // Reset State machine for next transmission
                    RFFull = 0x01; // Flag to let the main routine know we have a complete transmission
                    NOP();
                }
            }

            break;

        case TRFSYNC: // Expect header before looking for bits
            if (RFBit) 
            {   // rising edge detected  	+---+                  +---..
                //                              |   |  <-Theader-> |
                //                              +------------------+

                if ((RFcount < SHORT_HEAD) || (RFcount >= LONG_HEAD)) {
                    RFstate = TRFreset;
                    RFcount = 0;
                    break; // too short/long, no header
                } else {
                    RFcount = 0; // restart counter
                    RFstate = TRFUNO; // Header detected, start reading bits
                }
            } else { // still low
                RFcount++;
            }
            break;


        case TRFSYNC0: // Wait for high level before looking for synch header
            if (!RFsynch) // High level detected?
            {
                if (RFBit) // High level detected reset counters and wait for sync header
                {
                    RFsynch = 1;
                } //RFbit
            }//RFsynch
            else {
                if (!RFBit) // low level synch pulse is probably detected
                {
                    RFstate = TRFSYNC;
                    RFsynch = 0;
                }
            }
            break;


        case TRFreset: // Reset state machine
        default:
            RFstate = TRFSYNC0; // reset state machine in all other cases
            RFcount = 0;
            Bptr = 0;
            BitCount = 0;
            RFsynch = 0;
            break;

    } // switch
} // rxi

/**
 * Initialize KeeLoq bit-banged reception routines.
 * Configures TMR0 and reception state machine variables.
 */
void InitReceiver(void)
{
    TMR0 = 0;

    OPTION_REGbits.TMR0CS = 0;  // Clock from system clock

    INTCONbits.T0IF = 0;
    INTCONbits.T0IE = 1;        // Enable TMR1 overflow interrupt
    
    OPTION_REG = TIMER_CONFIG;
    //OPTION_REG = 0x0F; //

    RFstate = TRFreset; // reset state machine in all other cases
    RFFull = 0; // start with buffer empty
    XTMR = 0; // start extended timer
    RFtype = 0; // Type of incoming data (Keeloq, AES, XTEA)
}
